function freqPatterns = aprioriFPM(item_indices,datafile,sMin)
%-----------------------------------------------------------
% created by Jan Hrdlicka, 16.9.2010
%-----------------------------------------------------------
% Finds frequent itemsets with apriori algorithm and lexical sorting of the
% itemsets. Input is:

% "item_indices": Order of items (item indices) to be processed.
% Vector of size 1xM; where M is number of items

% "datafile": Transaction DB in a boolean matrix form. Size of the
% matrix is NxM, where N is number of transactions and M is number of items
% "sMin": Minimal relative support

words = {};
freqPatterns = {};
for i = item_indices
    word = i;
    if findSupport(word,datafile)>=sMin
        words{end+1,1} = word;
        words{end,2} = findSupport(word,datafile);
    end
end

while ~isempty(words)
    for i = 1:size(words,1)
        freqPatterns{end+1,1} = words{i,1};
        freqPatterns{end,2} = words{i,2};
    end
    wordCand = expandWords(words,item_indices);
    words = {};
    for i = 1:length(wordCand)
        s = findSupport(wordCand{i},datafile);
        if s>=sMin
            words{end+1,1} = wordCand{i};
            words{end,2} = s;
        end
    end
end


function supp = findSupport(word,datafile)
% Finds relative support of the transaction(itemset) "word" in transaction DB "datafile"
% in matrix form (without counts)
supp = datafile(:,word)==1;
supp = sum(all(supp==1,2))/size(datafile,1);

function nextWords = expandWords(words,item_indices)
% Generates next level words(itemsets).
% input is cell array "words" with words (arrays of items represented by numbers)
% and vector of sorted items "item_indices" with size 1xM, where M is number of
% items
nextWords = {};
for i = 1:size(words,1)
    actWord = words{i,1};
    ind = find(item_indices==actWord(end));
    if ind~=length(item_indices)
        for j = item_indices(ind+1:end)
            nextWords{end+1,1} = [actWord j];
        end
    end
end